import { AgoraAppStore, StreamType } from "any-hooks/types/agora";
import { onMounted, watch } from "vue";
import { useInjector, useState } from "vue-vulcan";
import { AGORA_APP_SDK } from "../communication/token";

interface AgoraStreamItem {
  url?: string;
  uid: number; //agora仅支持类型为number的uid
}

/** 基于声网sdk封装，由于多平台特性，请在应用根组件将对应平台的声网SDK对象以及appid通过useProvider提供; */
export function useAgoraClient() {  
  const [agoraErr, setError]            = useState(null);
  const [streamList, setStream]         = useState<AgoraStreamItem[]>([]);
  const { appid, client }               = useInjector<AgoraAppStore>(AGORA_APP_SDK);

  onMounted(() => {
    client.init(appid, () => {}, setError);
  })

  const joinChannelWithAgora = (token: string, cid: string, uid: number) => {
    console.log('准备加入视频频道 cid:',cid)
    client.join(
      token, 
      cid, 
      uid, 
      () => {pushVideoStream(); subscribeRemoteStream },
      setError
    );
  }

  //推送本地视频流
  const [localPushurl, setPushUrl]  = useState('');
  const pushVideoStream = () => {
    client.publish( url => {setPushUrl(url);console.log('成功生成推流地址'+url)});
  }

  //监听远程视频流
  const subscribeRemoteStream = () => {
    client.on('stream-added', evt => {      
      client.subscribe(
        evt.uid, 
        url => {updateStremList('add', {uid: evt.uid, url}); console.log(`用户${evt.uid}加入频道${url}`)},
        setError
      )
    });
    client.on('stream-removed', evt => updateStremList('remove',  evt.uid));
  }
  watch(streamList, val => console.log(`远端流变化val`, val));

  // 暂停本地流推送
  const muteLocalStream = (type: StreamType) => {
    return new Promise (resolve => {
      client.muteLocal(type, () => resolve(null))
    })    
  }

  // 恢复本地流推送
  const unmuteLocalStream = (type: StreamType) => {
    return new Promise (resolve => {
      client.unmuteLocal(type, () => resolve(null))
    })
  }

  // 暂停远程流接收
  const muteRemoteStream = (uid: number, type: StreamType) => {
    return new Promise( resolve => {
      client.mute(uid, type, () => resolve(null) )
    })
  }

  // 恢复远程流接收
  const unmuteRemoteStream = (uid: number, type: StreamType) => {
    return new Promise( resolve => {
      client.unmute(uid, type , () => resolve(null))
    })
  }

  // 根据uid来新增或移除远端视频流列表
  function updateStremList(act: 'add', input: {uid: number, url: string}): void;
  function updateStremList(act: 'remove', input: number[]): void;
  function updateStremList (action, input) {    
    switch(action) {
      case 'add':
        setStream(streamList.value.concat(input));
      break;
      case 'remove':
        console.log('remove', input)
        const res = streamList.value.filter( item => {
          !input.includes(item.uid)
        });
        setStream(res);
      break;
    }
  }

  // 退出频道
  const leaveChannelWithAgora = () => {
    return new Promise( resolve => {
      client.leave(() => resolve(''), setError);
    })
  }

  // 监听来自Agora Client的错误提示
  watch(agoraErr, err => {
    console.error(err)
  })

  return {
    localPushurl,
    streamList,
    joinChannelWithAgora,
    leaveChannelWithAgora,
    muteLocalStream,
    muteRemoteStream,
    unmuteLocalStream,
    unmuteRemoteStream
  }
}