import { useAuthData } from "any-hooks/auth/useAuthData";
import { watch } from "vue";
import { useInjector, useRequest, useState } from "vue-vulcan";
import { useNetSocketStore } from "./useNetSocketStore";

interface ChannelData {
  channel_id: string;
  agora_token: string;  
}

/** 频道中心功能，可以创建频道，并保存当前频道的信息，供呼叫中心、会议中心使用（目前同一时间仅允许存在一个频道） */
export function useChannelStore() {
  const [currentChannel, setChannel]   = useState<ChannelData>(null);
  const [channelMembers, setMembers]   = useState<string[]>([]);
  const [channelInfo, request]         = useRequest<ChannelData>('/getAgoraToken', {auto: false});  
  const { authData }                   = useInjector(useAuthData);
  const { currentMsg }                 = useInjector(useNetSocketStore);

  const createChannel = () => {
    return request();
  }  

  const getTokenByChannel = (cid: string) => {
    request({channel_id: cid});
  }  

  const updateMembers = (ids: string[], type: 'join' | 'leave') => {
    const current = channelMembers.value;
    if(type === 'join') {      
     setMembers([...new Set([...current, ...ids])])
    } else  {
      setMembers(current.filter(item => !ids.includes(item)))
    }    
  }  

  const clearChannel = () => {
    setChannel(null);
    setMembers([]);
  }

  watch(channelInfo, val => {    
    setChannel(val);    
    updateMembers([authData.value.id], 'join');
  })

  //监听频道下的成员实时信息
  watch(currentMsg, msg => {
    if(msg.msgMainFlag !== 'ChannelChat') return;
    switch(msg.msgSubFlag) {
      case 'Hangup':
        updateMembers([msg.fromID], 'leave');
      break;
    }
  })


  return {
    currentChannel,
    channelMembers,
    getTokenByChannel,
    createChannel,
    updateMembers,
    clearChannel
  }
}