import { setKeepScreenOn } from "@tarojs/taro";
import { useAuthData } from "any-hooks/auth/useAuthData";
import { useAgoraClient } from "any-hooks/common/useAgoraClient";
import { onMounted, watch } from "vue";
import { useInjector, useState } from "vue-vulcan";
import { useCallCenter } from "./useCallCenter";
import { useChannelStore } from "./useChannelStore";

/** 会议中心功能, 该功能在呼叫功能生效后使用，通过调用agoraSDK实现多人视频会议、语音会议的功能  */
export function useMeetingCenter() {
  const { currentChannel }  = useInjector(useChannelStore);
  const { authData }        = useInjector(useAuthData);
  const { hangup, myCallState }  = useInjector(useCallCenter);
  const {
    joinChannelWithAgora, 
    leaveChannelWithAgora,     
    muteLocalStream, 
    unmuteLocalStream,
    streamList, 
    localPushurl,
  } = useAgoraClient();
  

  /** 监听频道信息，有值或变更后将agora通讯功能加入频道 */
  watch(currentChannel, channel => {    
    if(!channel) return;
    joinChannelWithAgora(
      channel.agora_token,
      channel.channel_id,
      parseInt(authData.value.id)
    )
  })

  /** 离开会议(分两步：1.挂断当前呼叫， 2.退出agora音视频流) */
  const leave = () => {
    hangup(); 
    leaveChannelWithAgora();
  }

  /** 保持屏幕常亮  */
  onMounted(() => setKeepScreenOn({keepScreenOn: true}));

  /** 监听频道人数，人数为1时，自动退出频道  （目前直接用agora stream替代）*/
  watch(streamList, (current, last) => {    
    if(myCallState.value === 'free') return;
    if(current.length > last.length) return;
    if(current.length < 2) {
      setTimeout(leave, 3000)
    }
  })

  /** 切换麦克风使用状态  */
  const [voiceMute, setVoiceMute] = useState(false);
  const switchLocalMicState = () => {
    if(!voiceMute.value) {
      setVoiceMute(true)
      muteLocalStream('audio');
    } else {
      setVoiceMute(false);
      unmuteLocalStream('audio');
    }
  }

  return {
    streamList,
    localPushurl,
    voiceMute,
    leave,
    switchLocalMicState,
  }
}