import { UserData } from "../types/user";
import { useInjector, useRequest, useState } from "vue-vulcan";
import { computed, watch} from "vue";
import { useAuthData } from "../auth/useAuthData";
import { useNetSocketStore } from "any-hooks/communication/useNetSocketStore";

export function useContacts() {
  const { authData } = useInjector(useAuthData);
  const [ userList, getUserList ] = useRequest<UserData[]>('/getUserList', { auto: false });
  const { currentMsg } = useInjector(useNetSocketStore);   
  const [keyword, setKeyWord] = useState('');

  const contacts = computed( () => {
    return userList.value?.filter( item => item.id !== authData.value?.id )
                          .filter( item => item.nickname.includes(keyword.value))
  });

  const getContactById = (id: string) => {
    const index = userList.value.findIndex( user => user.id === id );
    return userList.value[index];
  }

  const getContactsByGroup = (groupId: string) => {
    return userList.value.filter( item => item.diy_group === groupId )
  }  

  // 监听socket的home类型消息 实时更新contacts联系人数据
  watch(currentMsg, msg => {
    if(msg.msgMainFlag !== 'Home') return;
    if(msg.msgSubFlag === 'UpdateUserList') getUserList();
  })

  return {
    contacts,
    setKeyWord,
    keyword,
    getUserList,
    getContactById,
    getContactsByGroup
  }
}