export type StreamType = 'video' | 'audio' | 'all';

export interface AgoraSdkClient {
  init: (appid: string, success?: any, err?: any) => void;
  join: (token: string, cid: string, uid: number, success?: any, err?: any) => void;
  rejoin: (cb: any) => void;
  publish(onSuccess: (url: string) => void, onFailure?: (err: { code: number; reason: string }) => void): void;
  on(event: "error", callback: (evt: { code: number; reason: string }) => void): void;
  on(event: "stream-added", callback: (evt: { uid: number, url?: string }) => void): void;
  on(event: "stream-removed", callback: (evt: { uid: number[] }) => void): void;
  off: (cb: any) => void;
  subscribe(uid: number, onSuccess: (url: string) => void, onFailure?: (err: any) => void): void;
  mute(uid: number, target: string, onSuccess?: () => void, onFailure?: (err: { code: number; reason: string }) => void): void;
  unmute(uid: number, target: string, onSuccess?: () => void, onFailure?: (err: { code: number; reason: string }) => void): void;
  muteLocal(target: StreamType, onSuccess?: () => void, onFailure?: (err: { code: number; reason: string }) => void): void;
  unmuteLocal(target: string, onSuccess?: () => void, onFailure?: (err: { code: number; reason: string }) => void): void;
  leave(onSuccess?: () => void, onFailure?: (err: { code: number; reason: string }) => void): void
  destory: (cb: any) => void;
}

export interface AgoraAppStore {
  appid: string;
  platform: 'web' | 'mini-app' | 'electron';
  client: AgoraSdkClient;
  stream?: any;
}
